﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;


using Csla;
using Csla.Data;
//using Csla.ApplicationContext;
using Library.Common.Logg;

namespace MGRptWebWisej
{
    public class ErrorLog
    {

        public enum UserActionType
        {
            NotDefined = 0,
            Insert = 1,
            Update = 2,
            Delete = 3
        }

        public static void Errormessage(Exception ex)
        {
            // ERROR: Not supported in C#: OnErrorStatement

            Errormessage(ex, "Feil oppstod", null);
        }

        public static void Errormessage(Exception ex, UserActionType UserActionType = UserActionType.NotDefined)
        {
            Errormessage(ex, "Feil under sletting", null, UserActionType);
        }

        public static void Errormessage(Exception ex, string Caption, object BusinessObject = null, UserActionType UserActionType = UserActionType.NotDefined)
        {
            // ERROR: Not supported in C#: OnErrorStatement

            //Hent INFO hvis noe
            string INFO = "";
            Exception inner = ex;

            while ((inner != null))
            {
                if (inner.Data.Contains("INFO"))
                {
                    if (!string.IsNullOrEmpty(INFO))
                    {
                        INFO = INFO + System.Environment.NewLine;
                    }
                    INFO = INFO + inner.Data["INFO"];
                }
                inner = inner.InnerException;
            }

            SqlException sqlException = null;
            inner = ex;
            while ((inner != null))
            {
                if (inner is SqlException)
                {
                    sqlException = (SqlException)inner;
                    break; // TODO: might not be correct. Was : Exit While
                }
                inner = inner.InnerException;
            }

            if (((BusinessObject != null)))
            {
                Csla.Rules.BrokenRulesTree brokenRulesTree = Csla.Rules.BusinessRules.GetAllBrokenRules(BusinessObject, true);
                if (((brokenRulesTree != null)))
                {
                    for (int i = 0; i <= brokenRulesTree.Count - 1; i++)
                    {
                        Csla.Rules.BrokenRulesNode brokenRulesNode = brokenRulesTree[i];
                        if (((brokenRulesNode.BrokenRules != null)) && brokenRulesNode.BrokenRules.Count > 0)
                        {
                            Csla.Rules.BrokenRule oneBrokenRule = brokenRulesNode.BrokenRules[0];
                            string FriendlyMessage = oneBrokenRule.Description;
                            DataPortal.Execute<LogErrorCommand>(new LogErrorCommand(ex, Caption, FriendlyMessage, ErrorType.UserError));
                            string M = null;
                            M = "Felter inneholder feil og du kan derfor ikke lagre endringer." + Environment.NewLine + Environment.NewLine + FriendlyMessage;
                            GeneralForms.MessageDialogForm.Start(Caption, M);
                            return;
                        }
                    }
                }
            }

            if (ex.ToString().ToUpper().Contains("edited by another user".ToUpper()))
            {
                string M = null;
                M = "Dataene har blitt endret av en annen bruker eller i et annet vindu. Du må lukke og hente inn data på nytt.";
                DataPortal.Execute<LogErrorCommand>(new LogErrorCommand(ex, Caption, M, ErrorType.UserError));
                GeneralForms.MessageDialogForm.Start(Caption, M);
                return;
            }
            else if (ex.ToString().Contains("Row has been deleted by another user"))
            {
                string M = null;
                M = "Dataene har blitt slettet av en annen bruker eller i et annet vindu. Du må hente inn data på nytt.";
                DataPortal.Execute<LogErrorCommand>(new LogErrorCommand(ex, Caption, M, ErrorType.UserError));
                GeneralForms.MessageDialogForm.Start(Caption, M);
                return;
            }
            else if (ex.ToString().Contains("[ERR_BR]"))
            {
                string M = null;
                Exception iex = ex.InnerException;
                while ((iex != null) && iex.Message.Substring(0,8) != "[ERR_BR]")
                {
                    iex = iex.InnerException;
                }
                if ((iex != null))
                {
                    M = iex.Message.Replace("[ERR_BR]", "");
                    DataPortal.Execute<LogErrorCommand>(new LogErrorCommand(ex, Caption, M, ErrorType.UserError));
                    GeneralForms.MessageDialogForm.Start(Caption, M);
                }
                else
                {
                    DataPortal.Execute<LogErrorCommand>(new LogErrorCommand(ex, Caption, ErrorType.ProgramError));
                    GeneralForms.MessageDialogForm.Start(Caption, RetreaveCompleteErrorMessage(ex));
                }
                return;
                //.ToString.Contains("Microsoft SQL Server, Error: 547") Then
            }
            else if (UserActionType == UserActionType.Delete && (sqlException != null) && sqlException.Number == 547)
            {
                string M = null;
                M = "Feil fra databasen indikerer at sletting ikke er mulig da posten du forsøker å slettet er i bruk.";
                DataPortal.Execute<LogErrorCommand>(new LogErrorCommand(ex, Caption, M, ErrorType.UserError));
                GeneralForms.MessageDialogForm.Start(Caption, M);
                return;
            }
            else
            {
                if (ex.Data.Contains("UserFault") && (string)ex.Data["UserFault"] == "YES")
                {
                    DataPortal.Execute<LogErrorCommand>(new LogErrorCommand(ex, Caption, ErrorType.UserError));
                }
                else
                {
                    DataPortal.Execute<LogErrorCommand>(new LogErrorCommand(ex, Caption, ErrorType.ProgramError));
                }
                GeneralForms.MessageDialogForm.Start(Caption, RetreaveCompleteErrorMessage(ex));

            }

            if (!string.IsNullOrEmpty(INFO))
            {
                GeneralForms.MessageDialogForm.Start("Info", INFO);
            }

        }

        private static string RetreaveCompleteErrorMessage(Exception ex) {
            string M = null;
            M = ex.Message + System.Environment.NewLine + System.Environment.NewLine + ex.StackTrace;
            if (ex.InnerException != null)
            {
                return M + System.Environment.NewLine + System.Environment.NewLine + RetreaveCompleteErrorMessage(ex.InnerException);
            }
            else
            {
                return M;
            }
        }

        public static void Errormessage(string Caption, string FriendlyMessage, object BusinessObject = null)
        {
            // ERROR: Not supported in C#: OnErrorStatement

            DataPortal.Execute<Library.Common.Logg.LogErrorCommand>(new LogErrorCommand(Caption, FriendlyMessage, ErrorType.UserError));
            GeneralForms.MessageDialogForm.Start(Caption, FriendlyMessage);
            return;
        }


    }
}